package Gear::Remotes;
use strict;
use warnings;
use Gear::Rules;
use Gear::Remotes::File;
use Gear::Remotes::GitUrl;

our $VERSION = '0.036';
our $verbose=0;

sub new {
    my $proto=shift;
    my $class=ref($proto) || $proto;
    my $self = {
	-remotename => 'upstream',
	@_,
    };
    $self->{-verbose} = $verbose if $verbose and (not $self->{-verbose} or $self->{-verbose} < $verbose);
    bless($self,$class);
    my $topdir=Gear::Git::Helper::get_git_toplevel();
    die "fatal: $topdir - not a gear repository.\n" unless &check_if_gear_dir($topdir);

    my $default_remotesname=$self->{-remotename};
    my $git_config_file=$topdir.'.git/config';
    my $git_remotes_obj=Gear::Remotes::File->new(-file => $git_config_file);
    my %in_git=map{$_=>1} $git_remotes_obj->list();

    my $gear_remotes_file=&get_gear_remotes_file($topdir);
    my $gear_remotes_obj=$self->{-remotes_obj};
    if (not $gear_remotes_obj and -e $gear_remotes_file) {
	$gear_remotes_obj=Gear::Remotes::File->new(-file => $gear_remotes_file);
    }
    if ($gear_remotes_obj) {
	my $gear_remotes_label=$gear_remotes_obj->label();
	my (@entries_in_git,@entries_out_git);
	my (%svn_remotes,%git_remotes);
	foreach my $entry ($gear_remotes_obj->list()) {
	    if ($in_git{$entry}) {
		push @entries_in_git, $entry;
	    } else {
		push @entries_out_git, $entry;
	    }
	    if ($entry=~/^(svn-)?remote\.([^\.]+)\.url=(.*)/) {
		my $svn=$1;
		my $remotesname=$2;
		my $url=$3;
		if ($url=~m,^[^:]+://git(?:ery)?(\.alt(?:linux\.(?:org|ru)?)?)[:/],) {
		    die "$gear_remotes_label: invalid remotes $entry: gitery.alt as upstream remotes makes no sense.\n";
		} else {
		    if ($svn) {
			$svn_remotes{$remotesname}=1;
		    } else {
			my $warning=&Gear::Remotes::GitUrl::warn_git_like_url($url);
			#warn 'W: ',$url,' -> ',($warning ? $warning:'');
			warn 'WARNING: there is a problem in ',$remotesname,' url [',$url,']: ', $warning,"\n" if $warning;
			$git_remotes{$remotesname}=1;
		    }
		}
	    }
	}

	if (@entries_out_git and not @entries_in_git) {
	    if ($self->{-autorestore}) {
		$gear_remotes_obj->append($git_config_file);
	    } else {
		die "Fatal: remotes listed in $gear_remotes_label are not present in $git_config_file. Run gear-remotes-restore first.\n";
	    }
	} elsif (@entries_out_git and @entries_in_git) {
	    if ($self->{-force_overwrite}) {
		$self->unset_all(@entries_in_git);
		@entries_in_git=();
		$gear_remotes_obj->append($git_config_file);
	    } else {
		print STDERR "entries in $gear_remotes_label and in $git_config_file\n";
		map {print STDERR "\t$_\n"} @entries_in_git;
		print STDERR "entries in $gear_remotes_label not in $git_config_file\n";
		map {print STDERR "\t$_\n"} @entries_out_git;
		die "Fatal: remotes in $gear_remotes_label conflicts with $git_config_file. Resolve conflict first.\n";
	    }
	}
	$self->{-git_remotes}=[sort keys (%git_remotes)];
	$self->{-svn_remotes}=[sort keys (%svn_remotes)];
    } elsif (grep {/^remote\.$default_remotesname\.url=./} keys(%in_git)) {
	$self->{-git_remotes}=[$default_remotesname];
	$self->{-svn_remotes}=[];
    } else {
	die 'Gear::Remotes: remotes not found: no ',$gear_remotes_file,' file and no [remote "',$default_remotesname,'"] entry in .git/config (use --remotesname <name> to override)', "\n";
    }
    return $self;
}

sub check_if_gear_dir {
    my $topdir=shift;
    my $test=-d $topdir.'.git' and (-d $topdir.'.gear' or -f $topdir.'.gear-rules');
    return $test;
}

sub get_gear_remotes_file {
    my $topdir=shift;
    my $gear_remotes_default=$topdir.'.gear/upstream/remotes';
    my $gear_remotes_file=$gear_remotes_default;
    $gear_remotes_file=$gear_remotes_default.'.'.$ENV{'USER'} if defined($ENV{'USER'}) and -e $gear_remotes_default.'.'.$ENV{'USER'};
    my $suffix=$ENV{'GEAR_UPSTREAM_REMOTES_SUFFIX'};
    $gear_remotes_file=$gear_remotes_default.'.'.$suffix if defined($suffix) and -e $gear_remotes_default.'.'.$suffix;
    return $gear_remotes_file;
}

sub fetch {
    my ($self)=@_;
    my $verbose=$self->{-verbose};
    my @git_svn_fetch_cmd=qw/git svn fetch/;
    my @git_fetch_cmd=qw/git fetch --recurse-submodules --tags --force/;
    if (! $verbose) {
	push @git_svn_fetch_cmd, '-q';
	push @git_fetch_cmd, '-q';
    }
    foreach my $remote (@{$self->{-svn_remotes}}) {
	print "git svn fetch $remote\n" if $verbose;
	system (@git_svn_fetch_cmd,$remote)==0 or die "ERROR: git-svn fetch $remote failed\n";
    }

    foreach my $remote (@{$self->{-git_remotes}}) {
	print "git fetch --recurse-submodules $remote\n" if $verbose;
	system (@git_fetch_cmd,$remote)==0 or die "ERROR: git fetch $remote failed\n";
    }
}

1;

__END__
