package Gear::Remotes::Tags;

use strict;
use warnings;
use Carp;
use Gear::Rules;
use IPC::Open2;

our $VERSION=0.003;
our $verbose=0;
my $debug=0;

#########################################
#		Private subs		#
#########################################

#9c0285c77a0a59e99ec638f91d3f32010309b03e     refs/tags/1.4.0
sub __remote2tags {
    return $_[0] =~ s,^[0-9a-fA-F]+\s+refs/tags/,,r;
}

sub __default_filter {
    return $_[0] =~ /^(?:[Vv]\.?)?\d[\d\.]*(?:-rel|-release)?$/;
}

sub __notimestamp_filter {
    return 0 if $_[0] =~ /^(?:199\d|20[0123]\d)(?:\d\d\d\d)?$/;
    return $_[0] =~ /^(?:[Vv]\.?)?\d[\d\.]*(?:-rel|-release)?$/;
}

sub __default_transform {
    my $tag=shift;
    $tag=~s/^[vV]\.?(\d)/$1/;
    $tag=~s/(?:\.|-rel|-release)$//;
    return $tag;
}

sub __process_args {
    my %args = (
	'-allow_external_command_exec' => 0,
	'-in_gear_repository' => 0,
	'-filter_sub' => undef,
	'-transform_sub' => \&__default_transform,
	@_
	);
    if ($args{'-filter_out_timestamps'}) {
	$args{'-filter_sub'}//= \&__notimestamp_filter;
    } else {
	$args{'-filter_sub'}//= \&__default_filter;
    }
    if ($args{'-in_gear_repository'}) {
	my $topdir=Gear::Git::Helper::get_git_toplevel();
	foreach my $bad_file ($topdir.'.gear/upstream/filter_tag',
			      $topdir.'.gear/upstream/transform_tag') {
	    if (-e $bad_file) {
		warn "WARNING: bad name for remotes: $bad_file. Should be *-tag, not *_tag.\n"
	    }
	}
	if ($args{'-allow_external_command_exec'}) {
	    $args{'-filter_cmd'}//= $topdir.'.gear/upstream/filter-tag';
	    $args{'-transform_cmd'}//= $topdir.'.gear/upstream/transform-tag';
	}
	$args{'-bantaglist'}//=$topdir.'.gear/upstream/bantaglist';
    }
    my $bantaglist=$args{'-bantaglist'};
    if (!$args{'-ban_map'} and $bantaglist and -r $bantaglist) {
	my %ban_map;
	open (my $fn, '<', $bantaglist) || die "Can't read $bantaglist: $!";
	while(<$fn>) {
	    s,\s*$,,;
	    s,^\s*,,;
	    next if /^\s*(?:#|$)/;
	    $ban_map{$_}=1;
	}
	close ($fn);
	$args{'-ban_map'}=\%ban_map;
    }
    return ($args{'-ban_map'}, $args{'-filter_sub'}, $args{'-transform_sub'}, $args{'-filter_cmd'}, $args{'-transform_cmd'});
}

# return a pair ($tag_pairs_ref, $exit_code)
sub __ls_tags_filtered {
    my ($ls_tags_cmd_ref, $preprocess_sub, $ban_map_ref, $filter_tag_sub, $transform_tag_sub, $filter_tag_file, $transform_tag_file)=@_;
    confess "internal error: expected ARRAY ref" unless $ls_tags_cmd_ref and ref($ls_tags_cmd_ref) eq 'ARRAY';
    my $has_filter_tag_cmd=(defined($filter_tag_file) and -x $filter_tag_file);
    my $has_transform_tag_cmd=(defined($transform_tag_file) and -x $transform_tag_file);
    my $ls_tags_exit_code = 0;
    my @git_tag_raw;
    open (my $ls_fn, '-|',@$ls_tags_cmd_ref) or die "can't run ",join(' ', @$ls_tags_cmd_ref),': ',$!;
    while (<$ls_fn>) {
	chomp;
	$_=&$preprocess_sub($_) if $preprocess_sub;
	next if $ban_map_ref and $ban_map_ref->{$_};
	push @git_tag_raw, $_;
    }
    close($ls_fn);
    if ($?) {
	$ls_tags_exit_code = $? >> 8;
	return undef, $ls_tags_exit_code;
    }
    if ($verbose>2) {
	print STDERR "DEBUG: run ",join(' ', @$ls_tags_cmd_ref),"\n";
	print STDERR "DEBUG: got: ",join(' ', @git_tag_raw),', got:',"\n";
    }
    my @git_tag;
    if ($has_filter_tag_cmd) {
	my($chld_out, $chld_in, $pid);
	$pid = open2($chld_out, $chld_in, $filter_tag_file);
	{
	    local $SIG{PIPE} = sub { die "filter_tag input pipe broke" };
	    print $chld_in map{ "$_\n" } @git_tag_raw;
	}
	close($chld_in) || die 'closing filter_tag input pipe ', $filter_tag_file, ' failed:', $!, ' exit=', $?;
	while (<$chld_out>) {
	    chomp;
	    push @git_tag, $_ if not m!^\s*$!;
	}
	close($chld_out) || die 'closing filter_tag output pipe ', $filter_tag_file, ' failed:', $!, ' exit=', $?;
	waitpid( $pid, 0 );
	my $child_exit_status = $? >> 8;
    } elsif ($filter_tag_sub) {
	@git_tag = grep {&$filter_tag_sub($_)} @git_tag_raw;
    } else {
	# allows V0.1 v.0.2.3 0.2-rel
	@git_tag = grep {/^(?:[Vv]\.?)?\d[\d\.]*(?:-rel|-release)?$/} @git_tag_raw;
    }
    my @git_tag_pairs;
    if ($has_transform_tag_cmd) {
	my($chld_out, $chld_in, $pid);
	$pid = open2($chld_out, $chld_in, $transform_tag_file);
	print $chld_in map {"$_\n"} @git_tag;
	close($chld_in);
	foreach my $tag (@git_tag) {
	    my $ver=<$chld_out>;
	    if (not defined($ver) or $ver=~/^\s*$/) {
		die "Gear::Remotes::Tags: bad filter $transform_tag_file: tried to transform $tag, got nothing";
	    }
	    chomp $ver;
	    push @git_tag_pairs, [$ver,$tag];
	}
	close($chld_out);
	waitpid( $pid, 0 );
	my $child_exit_status = $? >> 8;
    } elsif ($transform_tag_sub) {
	@git_tag_pairs = map {[&$transform_tag_sub($_),$_]} @git_tag;
    } else {
	foreach my $tag (@git_tag) {
	    my $ver=$tag;
	    $ver=~s/^[vV]\.?(\d)/$1/;
	    $ver=~s/(?:\.|-rel|-release)$//;
	    push @git_tag_pairs, [$ver,$tag];
	}
    }
    #warn "DEBUG: git_tag_pairs:\n", map{"$_->[0] -> $_->[1]\n"} @git_tag_pairs if $debug;
    return \@git_tag_pairs, $ls_tags_exit_code;
}



#########################################
#	Public static subs		#
#########################################

# return a pair ($tag_pairs_ref, $exit_code)
sub ls_remote_git_tags {
    my $url=shift;
    # to avoid authentification; fail immediately
    $url =~ s,^https?://,https://anonimous:anonimous@, if $url !~ m!^https?://[^/]*\@!;
    return &__ls_tags_filtered([qw/git ls-remote --tags/,$url], \&__remote2tags, &__process_args(@_));
}

my @ls_local_git_tags_cmd=qw/git tag/;

# return a pair ($tag_pairs_ref, $exit_code)
sub ls_local_git_tags_filtered {
    return &__ls_tags_filtered(\@ls_local_git_tags_cmd, undef, &__process_args(@_));
}

# used in RPM-Source-Update
sub find_local_tag_for_version {
    my ($version)=@_;
    my ($pairsref, $exit_code)=&ls_local_git_tags_filtered();
    die "Gear::Remotes::Tags: bad exit code from git tags command: ",$exit_code if $exit_code != 0;
    foreach my $pair (@$pairsref) {
	my ($ver,$tag)=@$pair;
	return $tag if $ver eq $version;
    }
    return;
}



# moved Versort from RPM::uscan to RPM::Devscripts::Versort
use RPM::Devscripts::Versort;

sub cmp_tag_versions {
    my ($version, $tag_pairs_ref)=@_;
    confess "Gear::Remotes: internal error: expected ARRAY ref in 2nd arg"
	unless $tag_pairs_ref and ref($tag_pairs_ref);
    my @sorted = RPM::Devscripts::Versort::upstream_versort(@$tag_pairs_ref);
    my $max_tagver=$sorted[0]->[0];
    my $version_cmp=RPM::Devscripts::Versort::trimmed_vercmp($version, $max_tagver);
    return ($version_cmp, $version, $max_tagver, $sorted[0]->[1]);
}


1;

__END__
